#include "theappmain.h"
#include "ui_theappmain.h"

TheAppMain::TheAppMain(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::TheAppMainClass)
{
    ui->setupUi(this);

    status = new QLabel("...");
            
    ui->statusBar->addWidget(status, 0);

    connect(ui->action_Quit, SIGNAL(triggered()), this, SLOT(on_bQuit_clicked()));
    connect(ui->actionRefresh, SIGNAL(triggered()), this, SLOT(on_bRefresh_clicked()));
    connect(ui->actionRun_all, SIGNAL(triggered()), this, SLOT(on_bRunAll_clicked()));
    connect(ui->action_Configuration, SIGNAL(triggered()), this, SLOT(on_aConfiguration_triggered()));

    QCoreApplication::setOrganizationName("LiNC");
    QCoreApplication::setOrganizationDomain("defcon.no");
    QCoreApplication::setApplicationName("Huset Personalkort");


    while ( setupsql() == false ) {
        qDebug() << "Error initializing database.";
        status->setText("Error initializing database.");
    }

}

TheAppMain::~TheAppMain()
{
    delete ui;
}

bool TheAppMain::setupsql() {

    // If problems setting up MySQL drivers on Windows is encountered, I had great
    // success following the guide at http://www.jiggerjuice.net/software/qt-sql-drivers.html
    //
    // To summarize the steps, assuming the following paths:
    //    C:\Qt\2009.2\  <- Install dir of the Qt SDK
    //    C:\MySQL5\     <- Install dir of MySQL essentials
    //    C:\mingw-utils <- Install dir of the MinGW-Utils 0.3 pack
    //
    // 1) Install MySQL Essentials, removing the Server component itself is optional
    //    For sake of simplicity, install it to C:\mysql or C:\mysql5
    // 2) Install the MinGW utils package (current 0.3.tar.gz)
    // 3) Make the libmysql.lib in the MySQL install lib-compatible with mingw:
    //    cd c:\mysql5\lib\opt
    //    c:\mingw-utils\bin\reimp.exe -d libmysql.lib
    //    c:\mingw-utils\bin\dlltool.exe -k --input-def libmysql.def --dllname libmysql.dll --output-lib libmysql.a
    // 4) Configure the QtSQL MySQL driver:
    //    cd c:\Qt\2009.2\src\plugins\sqldrivers\mysql
    //    qmake -o Makefile "INCLUDEPATH+=C:\MYSQL5\INCLUDE"  "LIBS+=-LC:\MYSQL5\LIB\OPT -lmysql"  mysql.pro
    // 5) Compile the QtSQL MySQL driver:
    //    mingw32-make clean
    //    mingw32-make
    // 6) Install the QtSQL MySQL driver:
    //    mingw32-make install
    // 7) Remember to copy libmySQL.dll from C:\mysql5\bin to somewhere in your build-configuration's path,
    //    e.g. your app's working directory (e.g. the debug or release dir) or your windows\system32..

    bool ok;

    if ( db.isOpen() ) {
        // FIXME: Database disconnect/close is _buggy_.
        // QSqlDatabasePrivate::removeDatabase: connection 'qt_sql_default_connection' is still in use, all queries will cease to work.
        // QSqlDatabasePrivate::addDatabase: duplicate connection name 'qt_sql_default_connection', old connection removed.
        // Need to figure out how to get rid of these.
        qDebug() << "Closing open Database";
        status->setText("Closing open Database");
        model->clear();
        delete model;
        db.removeDatabase("qt_sql_default_connection");
        db.close();
    }

    // The readsettings() call gets configuration data, and fills the
    // relevant class member variables.
    readsettings();

    // If configuration data is available, we are ready to continue
    // setting up SQL connection.
    if ( ! (
            sqlhostname == ("") ||
            sqlusername == ("") ||
            sqlpassword == ("") ||
            sqldatabase == ("") ||
            (sqlportnumber == 0) )
    ) {

        // Debug output. Compiling and installing MySQL drivers on windows
        // has proven to be such a headache, that we really need as much
        // debug output relating to installed SQL drivers as possible.
        // The following output listing of available drivers is yanked from
        // the Qt documentation
        QStringList list = QSqlDatabase::drivers();
        QStringList::Iterator it = list.begin();
        qDebug() << "Available SQL drivers:";
        while( it != list.end() ) {
            QString drivername = *it;
            qDebug() << "   [" << drivername << "]";
            ++it;
        }

        if ( db.isDriverAvailable("QMYSQL")) {
            db = QSqlDatabase::addDatabase("QMYSQL");
        } else {
            QMessageBox::critical(this,
            "Database driver error.",
            "The Qt MySQL driver is not available.\nUnable to continue");
            exit(1);
        }
        db.setHostName(sqlhostname);
        db.setDatabaseName(sqldatabase);
        db.setUserName(sqlusername);
        db.setPassword(sqlpassword);
        db.setPort(sqlportnumber);
        ok = db.open();

    } else { ok = false; }

    if ( !ok) {
        qDebug() << "Unable to connect to database " << sqldatabase << " on server " << sqlhostname;
        if ( on_aConfiguration_triggered() == false ) {
            qDebug() << "Application not configured, and configuration cancelled.";
            exit(1);
        }
        return false;
    }

    sqlquerystring = "SELECT  personer.`id` as id, idkort.`id` as kortid , CONCAT( `fornavn` , ' ', `etternavn` ) AS `name` , `fornavn` , `etternavn` , `bilde`, `strekkode` FROM personer INNER JOIN idkort ON personer.id = idkort.personid WHERE idkort.ok = 1";

    const QSqlQuery query (sqlquerystring);
    model = new QSqlTableModel;
    model->setQuery(query);
    ui->personList->setModel(model);
    ui->personList->setModelColumn(2);
    status->setText("OK. Attached to database, data ready.");
    return true;
}

void TheAppMain::readsettings() {
    QSettings settings;
    sqlhostname   = settings.value("sqlhostname",   "").toString();
    sqlusername   = settings.value("sqlusername",   "").toString();
    sqlpassword   = settings.value("sqlpassword",   "").toString();
    sqldatabase   = settings.value("sqldatabase",   "").toString();
    sqlportnumber = settings.value("sqlportnumber", 0 ).toInt();
    qDebug() << "Read settings: " << sqlhostname << " " << sqlusername << " " << sqlpassword << " " << sqldatabase << " " << sqlportnumber;

}

void TheAppMain::writesettings() {

    qDebug() << "Write settings requested.";
    qDebug() << "Current variable content: " << sqlhostname << " " << sqlusername << " " << sqlpassword << " " << sqldatabase << " " << sqlportnumber;

    if (
            sqlhostname == ("") ||
            sqlusername == ("") ||
            sqlpassword == ("") ||
            sqldatabase == ("") ||
            (sqlportnumber == 0)
    ) {
        qDebug() << "One or more variable not set, aborting write.";
        return;
    }
    QSettings settings;
    settings.setValue("sqlhostname",   sqlhostname);
    settings.setValue("sqlusername",   sqlusername);
    settings.setValue("sqlpassword",   sqlpassword);
    settings.setValue("sqldatabase",   sqldatabase);
    settings.setValue("sqlportnumber", sqlportnumber);
    qDebug() << "Trying to run sync() on settings.";
    settings.sync();

}

void TheAppMain::on_bPreviewSelected_clicked()
{
    int i = ui->personList->currentIndex().row();
    if ( i == -1 )
    {
        qDebug() << "No selection made.";
        status->setText("No selection made.");
        return;
    }
    int id            = model->record(i).value("id").toInt();
    QString name      = model->record(i).value("name").toString();
    QString firstname = model->record(i).value("fornavn").toString();
    QString lastname  = model->record(i).value("etternavn").toString();
    QString image     = model->record(i).value("bilde").toString();
    QString barcode   = model->record(i).value("strekkode").toString();
    qDebug() << "Row # " << i << "has ID: " << id << " and name: " << name;

    Preview *prev = new Preview(this, id, name, firstname, lastname, image, barcode);
    prev->exec();
    qDebug() << "Returned from preview...";
    status->setText("No selection made.");

}

void TheAppMain::on_bQuit_clicked()
{
    delete model;
    db.close();
    this->close();
}

void TheAppMain::on_bRunAll_clicked()
{
    int numrows = model->rowCount();
    qDebug() << numrows << "Rows in query result";
    if ( numrows < 1 )
    {
        qDebug() << "No cards to generate.";
        status->setText("No cards to generate.");
        return;
    }
    ui->bRefresh->setDisabled(true);
    ui->bPreviewSelected->setDisabled(true);
    ui->bRunSelected->setDisabled(true);
    ui->bRunAll->setDisabled(true);

    QPrinter printer( QPrinter::HighResolution );
    QPrintDialog printDialog ( &printer, this );

    QString documenttitle = "ID Card bulk job, ";
    documenttitle.append( QString().setNum( numrows) );
    documenttitle.append(" cards");
    printer.setDocName(documenttitle);

    if ( printDialog.exec() )
    {
        QPainter painter ( &printer );

        for ( int i = 0; i < numrows; i++ )
        {
            if ( i != 0 ) {  printer.newPage(); }

            int id            = model->record(i).value("id").toInt();
            int cardid        = model->record(i).value("kortid").toInt();
            QString name      = model->record(i).value("name").toString();
            QString firstname = model->record(i).value("fornavn").toString();
            QString lastname  = model->record(i).value("etternavn").toString();
            QString image     = model->record(i).value("bilde").toString();
            QString barcode   = model->record(i).value("strekkode").toString();
            qDebug() << "Row # " << i << "has ID: " << id << " and name: " << name;

            QRect rect = painter.viewport();
            QSize page_size = rect.size();
            int x = page_size.width();
            int y = page_size.height();
            qDebug() << "Print dialog accepted. Page size available is: " << x << " x " << y;

            // Create an ID card image, then transform it from "Landscape"
            // to "Portrait" direction, before dumping it onto the printer.
            IDCardImage tmp_image(y, x, id, name, firstname, lastname, image, barcode);
            QMatrix m; m.rotate( 90.0 );
            QImage t_image = tmp_image.transformed(m);

            // Scale the ID card to the size of the printer canvas,
            // befre draing the image onto the printer painter.
            QSize size = t_image.size();
            size.scale(rect.size(), Qt::KeepAspectRatio);
            painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
            painter.setWindow(t_image.rect());
            painter.drawImage(0,0, t_image);

            // After painting is complete,we are ready to print.

            // The actual printing is done when the QPainter reaches
            // its end-of-life, and no errors have occurred.
            if ( db.isOpen() ) {
                QString sqlupdate = "DELETE FROM idkort WHERE idkort.id = ";
                sqlupdate.append(QString().setNum( cardid ));
                qDebug() << "SQL delete string: " << sqlupdate;
                db.exec(sqlupdate);
            }
        }
    }
    setupsql();
    ui->bRefresh->setDisabled(false);
    ui->bPreviewSelected->setDisabled(false);
    ui->bRunSelected->setDisabled(false);
    ui->bRunAll->setDisabled(false);
}

void TheAppMain::on_bRefresh_clicked()
{
//    const QSqlQuery query (sqlquerystring);
//    delete model;
//    model = new QSqlTableModel;
//    model->setQuery(query);
//    ui->personList->setModel(model);
//    ui->personList->setModelColumn(2);
    setupsql();
}

void TheAppMain::on_personList_doubleClicked(QModelIndex index)
{
    this->on_bPreviewSelected_clicked();
}

bool TheAppMain::on_aConfiguration_triggered()
{
    SettingsDialog * dialog = new SettingsDialog(this);
    qDebug() << "Opening settings dialog.";
    if ( dialog->exec() ) {
        sqlhostname  = dialog->gethostname();
        sqlusername  = dialog->getusername();
        sqlpassword  = dialog->getpassword();
        sqldatabase =  dialog->getdatabase();
        sqlportnumber  = dialog->getportnumber();
        qDebug() << sqlhostname << " " << sqlusername << " " << sqlpassword << " " << sqldatabase << " " << sqlportnumber;
        writesettings();
        delete dialog;
        return true;

    } else {
        qDebug() << "User selected to cancel configuration...";
        delete dialog;
        return false;
    }
}

void TheAppMain::on_bRunSelected_clicked()
{
    qDebug() << "yey";
    int i = ui->personList->currentIndex().row();
    if ( i == -1 )
    {
        qDebug() << "No selection made.";
        status->setText("No selection made.");
        return;
    }
    int id            = model->record(i).value("id").toInt();
    int cardid        = model->record(i).value("kortid").toInt();
    QString name      = model->record(i).value("name").toString();
    QString firstname = model->record(i).value("fornavn").toString();
    QString lastname  = model->record(i).value("etternavn").toString();
    QString image     = model->record(i).value("bilde").toString();
    QString barcode   = model->record(i).value("strekkode").toString();
    qDebug() << "Row # " << i << "has ID: " << id << " and name: " << name;

    QPrinter printer( QPrinter::HighResolution );
    QPrintDialog printDialog ( &printer, this );
    
    QString documenttitle = "ID Card for ";
    documenttitle.append(name);
    printer.setDocName(documenttitle);

    if ( printDialog.exec() )
    {
        QPainter painter ( &printer );
        QRect rect = painter.viewport();
        QSize page_size = rect.size();
        int x = page_size.width();
        int y = page_size.height();
        qDebug() << "Print dialog accepted. Page size available is: " << x << " x " << y;

        // Create an ID card image, then transform it from "Landscape"
        // to "Portrait" direction, before dumping it onto the printer.
        IDCardImage tmp_image(y, x, id, name, firstname, lastname, image, barcode);
        QMatrix m; m.rotate( 90.0 );
        QImage t_image = tmp_image.transformed(m);

        // Scale the ID card to the size of the printer canvas,
        // befre draing the image onto the printer painter.
        QSize size = t_image.size();
        size.scale(rect.size(), Qt::KeepAspectRatio);
        painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
        painter.setWindow(t_image.rect());
        painter.drawImage(0,0, t_image);

        // After painting is complete,we are ready to print.
        // The actual printing is done when the QPainter reaches
        // its end-of-life, and no errors have occurred.
        if ( db.isOpen() ) {
            QString sqlupdate = "DELETE FROM idkort WHERE id = ";
            sqlupdate.append(QString().setNum( cardid ));
            qDebug() << "SQL delete string: " << sqlupdate;
            db.exec(sqlupdate);
        }
        setupsql();
    }
}
